//==============================================================================
//File name:    "startup.cpp"
//Purpose:      Source File
//Version:      1.00
//Copyright:    (c) 2017, Akimov Vladimir  E-mail: decoder@rambler.ru	
//==============================================================================
#include "startup.h"
#include <ioCC2540.h>
#include <intrinsics.h>
#include "ioCCxx10_bitdef.h"
#include "ioCC2540_bitdef.h"

extern unsigned int System_Timer;

//------------------------------------------------------------------------------
//Function: System Initialisation and Launch
//------------------------------------------------------------------------------
void System_StartUp(void)
{
  //Disable global interrupt by setting the [IEN0.EA=0] (EA = 0;)
  __disable_interrupt();
  
  //Ports I/O initialization
  //ports_ini();        
  
  //Reset
  System_Timer = 0; 

  //------------------------------------------
  //CC2540 have 4 oscillators 
  //1. 32MHz external Crystal 
  //2. 16MHz internal RC 
  //3. 32KHz external Crystal
  //4. 32KHz internal RC
  
  //Goto to active mode from sleep, Active or Idle mode
  CLKCONCMD &= 0xFC; //
    
  //32-kHz internal RC OSC clock-source select for Sleep Timer + WDT.
  CLKCONCMD |= CLKCONCMD_OSC32K;
  
  //Calibration oscillator enable.
  SLEEPCMD &= (~OSC32K_CALDIS)|(0xFC);
  
  //Wait until the 32KHz internal RC is stable.
  while(!(SLEEPSTA & CLK32K));

  //Change the system clock source 0: 32 MHz XOSC.
  CLKCONCMD &= ~CLKCONCMD_OSC;
  
  //Wait until source has actually changed.
  //0: 32-MHz XOSC selekted OK
  while(CLKCONSTA & CLKCONSTA_OSC);
  
  //Power down the HS RCOSC, since it is not beeing used.
  //Note that the HS RCOSC should not be powered down before the applied
  //system clock source is stable (SLEEP.XOSC_STB = 1).
  ///SLEEPCMD |= SLEEP_OSC_PD;

  //Clear Timer 1 overflow interrupt flag
  //CPU interrupt flag (IRCON) for Timer 1 is cleared automatically by hardware
  ///T1CTL &= ~T1CTL_OVFIF;  !
  /*
  //Disable interrupt for the other channels
  T1CCTL0 &= ~T1CCTL0_IM;   // Disable interrupt on channel 0
  T1CCTL1 &= ~T1CCTL1_IM;   // Disable interrupt on channel 1
  T1CCTL2 &= ~T1CCTL2_IM;   // Disable interrupt on channel 2

  T1CTL = T1CTL_DIV_128 | T1CTL_MODE_FREERUN;   // 646ms
  
  //Set interrupt enable for overflow on Timer 1 and 
  T1OVFIM = 1; 
  
  //Clear Timer counter
  T1CNTL = 0;
  T1CNTH = 0;
   
  //Enable interrupt by setting IEN1.T1IE
  T1IE = 1; */
  
  //Output direction
  P0DIR =(1<<1); 
  
  //====== Ini System Timer counter ==================
  T4CTL = 0;
  T4CNT = 0;
  T4CCTL0 &= ~T4CCTL0_IM;   //Disable interrupt
  T4CCTL1 &= ~T4CCTL1_IM;   //Disable interrupt

  //System timer
  T4CTL = T4CTL_DIV_128 | T4CTL_START | T4CTL_OVFIM | 
          T4CTL_CLR | T4CTL_MODE_FREERUN; // 2.52ms
  
  //Enable interrupt by setting IEN1.T1IE
  T4IE = 1;
  //===================================================

  
  //Initialise device setup 
  ///Device_Setup_ini();    
  
  //LED_GRN_OFF(); 
  
  __enable_interrupt();   
}
