//******************************************************************************
//*
//*     FULLNAME:  Single-Chip Microcontroller Real-Time Operating System
//*
//*     NICKNAME:  scmRTOS
//*
//*     PROCESSOR: AVR (Atmel)
//*
//*     TOOLKIT:   EWAVR (IAR Systems)
//*
//*     PURPOSE:   Project Level Target Extensions Config
//*
//*     Version: 4.00
//*
//*     $Revision: 529 $
//*     $Date:: 2012-04-04 #$
//*
//*     Copyright (c) 2003-2012, Harry E. Zhurov
//*
//*     Permission is hereby granted, free of charge, to any person 
//*     obtaining  a copy of this software and associated documentation 
//*     files (the "Software"), to deal in the Software without restriction, 
//*     including without limitation the rights to use, copy, modify, merge, 
//*     publish, distribute, sublicense, and/or sell copies of the Software, 
//*     and to permit persons to whom the Software is furnished to do so, 
//*     subject to the following conditions:
//*
//*     The above copyright notice and this permission notice shall be included 
//*     in all copies or substantial portions of the Software.
//*
//*     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
//*     EXPRESS  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
//*     MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
//*     IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
//*     CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
//*     TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
//*     THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//*
//*     =================================================================
//*     See http://scmrtos.sourceforge.net for documentation, latest
//*     information, license and contact details.
//*     =================================================================
//*
//******************************************************************************

#ifndef  scmRTOS_TARGET_CFG_H
#define  scmRTOS_TARGET_CFG_H


//#include <ioCC2510.h>
//#include <ioCC2530.h>
#include <ioCC2540.h>
#include <ioCCxx10_bitdef.h>

//------------------------------------------------------------------------------
// Virtual register
//------------------------------------------------------------------------------

#define VREG_OFFSET (BANK_OFFSET+REG_COUNT)

#define V0  VREG_OFFSET
#define V1  VREG_OFFSET+1
#define V2  VREG_OFFSET+2
#define V3  VREG_OFFSET+3
#define V4  VREG_OFFSET+4
#define V5  VREG_OFFSET+5
#define V6  VREG_OFFSET+6
#define V7  VREG_OFFSET+7
#define V8  VREG_OFFSET+8
#define V9  VREG_OFFSET+9
#define V10 VREG_OFFSET+10
#define V11 VREG_OFFSET+11
#define V12 VREG_OFFSET+12
#define V13 VREG_OFFSET+13
#define V14 VREG_OFFSET+14
#define V15 VREG_OFFSET+15
#define V16 VREG_OFFSET+16
#define V17 VREG_OFFSET+17
#define V18 VREG_OFFSET+18
#define V19 VREG_OFFSET+19
#define V20 VREG_OFFSET+20
#define V21 VREG_OFFSET+21
#define V22 VREG_OFFSET+22
#define V23 VREG_OFFSET+23
#define V24 VREG_OFFSET+24
#define V25 VREG_OFFSET+25
#define V26 VREG_OFFSET+26
#define V27 VREG_OFFSET+27
#define V28 VREG_OFFSET+28
#define V29 VREG_OFFSET+29
#define V30 VREG_OFFSET+30
#define V31 VREG_OFFSET+31

#define SPXH (BANK_OFFSET+REG_COUNT+VREG_COUNT+1)
#define SPXL (BANK_OFFSET+REG_COUNT+VREG_COUNT)

//------------------------------------------------------------------------------
// Interrupt Vectors
//------------------------------------------------------------------------------

#undef  RFTXRX_VECTOR
#undef  ADC_VECTOR
#undef  URX0_VECTOR
#undef  URX1_VECTOR
#undef  ENC_VECTOR
#undef  ST_VECTOR
#undef  P2INT_VECTOR
#undef  UTX0_VECTOR
#undef  DMA_VECTOR
#undef  T1_VECTOR
#undef  T2_VECTOR
#undef  T3_VECTOR
#undef  T4_VECTOR
#undef  P0INT_VECTOR
#undef  UTX1_VECTOR
#undef  P1INT_VECTOR
#undef  RF_VECTOR
#undef  WDT_VECTOR

#define  RFTXRX_VECTOR  VECT(  0, 0x03+2 )   /*  RF TX done / RX ready                       */
#define  ADC_VECTOR     VECT(  1, 0x0B+2 )   /*  ADC End of Conversion                       */
#define  URX0_VECTOR    VECT(  2, 0x13+2 )   /*  USART0 RX Complete                          */
#define  URX1_VECTOR    VECT(  3, 0x1B+2 )   /*  USART1 RX Complete                          */
#define  ENC_VECTOR     VECT(  4, 0x23+2 )   /*  AES Encryption/Decryption Complete          */
#define  ST_VECTOR      VECT(  5, 0x2B+2 )   /*  Sleep Timer Compare                         */
#define  P2INT_VECTOR   VECT(  6, 0x33+2 )   /*  Port 2 Inputs                               */
#define  UTX0_VECTOR    VECT(  7, 0x3B+2 )   /*  USART0 TX Complete                          */
#define  DMA_VECTOR     VECT(  8, 0x43+2 )   /*  DMA Transfer Complete                       */
#define  T1_VECTOR      VECT(  9, 0x4B+2 )   /*  Timer 1 (16-bit) Capture/Compare/Overflow   */
#define  T2_VECTOR      VECT( 10, 0x53+2 )   /*  Timer 2 (MAC Timer) Overflow                */
#define  T3_VECTOR      VECT( 11, 0x5B+2 )   /*  Timer 3 (8-bit) Capture/Compare/Overflow    */
#define  T4_VECTOR      VECT( 12, 0x63+2 )   /*  Timer 4 (8-bit) Capture/Compare/Overflow    */
#define  P0INT_VECTOR   VECT( 13, 0x6B+2 )   /*  Port 0 Inputs                               */
#define  UTX1_VECTOR    VECT( 14, 0x73+2 )   /*  USART1 TX Complete                          */
#define  P1INT_VECTOR   VECT( 15, 0x7B+2 )   /*  Port 1 Inputs                               */
#define  RF_VECTOR      VECT( 16, 0x83+2 )   /*  RF General Interrupts                       */
#define  WDT_VECTOR     VECT( 17, 0x8B+2 )   /*  Watchdog Overflow in Timer Mode             */


#if scmRTOS_CONTEXT_SWITCH_SCHEME == 1

#define CONTEXT_SWITCH_ISR_VECTOR  WDT_VECTOR

#endif


#ifndef __IAR_SYSTEMS_ASM__

#if scmRTOS_CONTEXT_SWITCH_SCHEME == 1

#define SPM_CONTROL_REG IEN2

#endif


//------------------------------------------------------------------------------
//
//       System Timer stuff
//
//
#define SYSTEM_TIMER_VECTOR T4_VECTOR
#define LOCK_SYSTEM_TIMER()  (T4CTL&= ~T4CTL_START)
#define UNLOCK_SYSTEM_TIMER() (T4CTL|= T4CTL_START)

namespace OS
{
    #pragma vector=SYSTEM_TIMER_VECTOR
    OS_INTERRUPT void OS_SystemTimer_ISR(void);
}
//------------------------------------------------------------------------------
//
//       Context Switch ISR stuff
//
//
namespace OS
{

#if scmRTOS_CONTEXT_SWITCH_SCHEME == 1

  INLINE void raise_context_switch() 
    {IRCON2|=(1<<4); SPM_CONTROL_REG |= IEN2_WDTIE;} // enable SPM interrupt
  INLINE void block_context_switch() 
    {IRCON2&=~(1<<4); SPM_CONTROL_REG &= ~IEN2_WDTIE;} // disable SPM interrupt

#endif // scmRTOS_CONTEXT_SWITCH_SCHEME
}
//------------------------------------------------------------------------------
#else  // __IAR_SYSTEMS_ASM__

#endif // __IAR_SYSTEMS_ASM__


#endif // scmRTOS_TARGET_CFG_H
//------------------------------------------------------------------------------

